﻿Public Class TreeNode(Of T)
    ' This node's data.
    Public Property Data As T

    ' The node's position.
    Public Property X As Integer
    Public Property Y As Integer

    ' This node's children.
    Private Children As List(Of TreeNode(Of T)) = New List(Of TreeNode(Of T))()

    ' Constructor.
    Public Sub New(ByVal newData As T, ByVal newX As Integer, ByVal newY As Integer)
        Data = newData
        X = newX
        Y = newY
    End Sub

    ' Override ToString to display the data.
    Public Overrides Function ToString() As String
        If Data Is Nothing Then Return ""
        Return Data.ToString()
    End Function

    ' Add a piece of data as a child.
    Public Function AddChild(ByVal newData As T, ByVal newX As Integer, ByVal newY As Integer) As TreeNode(Of T)
        Dim child As TreeNode(Of T) = New TreeNode(Of T)(newData, newX, newY)
        Children.Add(child)
        Return child
    End Function

    ' Draw the node's subtree links.
    Public Sub DrawLinks(ByVal gr As Graphics)
        ' Recursively draw the children.
        For Each child As TreeNode(Of T) In Children
            ' Draw the child.
            child.DrawLinks(gr)

            ' Draw a line to the child.
            gr.DrawLine(Pens.Blue, X, Y, child.X, child.Y)
        Next child
    End Sub

    ' Draw the node itself.
    Public Sub DrawNode(ByVal gr As Graphics, ByVal theFont As Font)
        ' Recursively draw the children.
        For Each child As TreeNode(Of T) In Children
            ' Draw the child.
            child.DrawNode(gr, theFont)
        Next child

        ' Draw this node.
        Using string_format As New StringFormat()
            ' Draw a circle.
            gr.FillEllipse(Brushes.White, X - 10, Y - 10, 20, 20)
            gr.DrawEllipse(Pens.Black, X - 10, Y - 10, 20, 20)

            ' Draw the text.
            string_format.Alignment = StringAlignment.Center
            string_format.LineAlignment = StringAlignment.Center
            gr.DrawString(ToString(), theFont, Brushes.Green, X, Y, string_format)
        End Using
    End Sub

    ' Draw this node's subtree. Do this for the root node.
    Public Sub Draw(ByVal gr As Graphics, ByVal theFont As Font)
        DrawLinks(gr)
        DrawNode(gr, theFont)
    End Sub
End Class
